import React, { useState, useEffect, useRef, useMemo, Fragment, useCallback, useImperativeHandle } from 'react';
import styled, { keyframes, css, ThemeProvider } from 'styled-components';
import { areEqual, FixedSizeList } from 'react-window';

const _bounce_keyframes = keyframes`
  0%, 80%, 100% {
    transform: scale(0);
  } 40% {
    transform: scale(1.0);
  }
`;

const _fade_in_keyframes = keyframes`
  from {
    opacity: 0;
  } to {
    opacity: 1;
  }
`;

const BOUNCE_ANIMATION_CSS = css`${_bounce_keyframes} 1.19s ease-in-out infinite`;
const FADE_IN_ANIMATION_CSS = css`${_fade_in_keyframes} 0.225s ease-in-out forwards`;

var DefaultThemeObj = Object.freeze({
  color: {
    border: '#ced4da',
    danger: '#dc3545',
    primary: '#007bff',
    disabled: '#e9ecef',
    placeholder: '#6E7276',
    dangerLight: 'rgba(220, 53, 69, 0.25)'
  },
  input: {},
  select: {},
  loader: {
    size: '0.625rem',
    padding: '0.375rem 0.75rem',
    animation: BOUNCE_ANIMATION_CSS,
    color: 'rgba(0, 123, 255, 0.42)'
  },
  icon: {
    color: '#cccccc',
    hoverColor: '#A6A6A6',
    padding: '0 0.9375rem',
    clear: {
      fontWeight: 900,
      fontSize: '0.85em',
      animation: FADE_IN_ANIMATION_CSS,
      transition: 'color 0.15s ease-in-out',
      fontFamily: 'Helvetica, Arial, sans-serif'
    },
    caret: {
      size: '7px',
      transition: 'transform 0.225s ease-in-out, color 0.15s ease-in-out'
    }
  },
  control: {
    minHeight: '38px',
    borderWidth: '1px',
    borderStyle: 'solid',
    borderRadius: '0.25rem',
    boxShadow: '0 0 0 0.2rem',
    padding: '0.375rem 0.75rem',
    boxShadowColor: 'rgba(0, 123, 255, 0.25)',
    focusedBorderColor: 'rgba(0, 123, 255, 0.75)',
    transition: 'box-shadow 0.15s ease-in-out, border-color 0.15s ease-in-out'
  },
  menu: {
    width: '100%',
    margin: '0.5rem 0',
    padding: '0.15rem 0',
    borderRadius: '0.25rem',
    backgroundColor: '#fff',
    animation: FADE_IN_ANIMATION_CSS,
    boxShadow: '0 0 0 1px rgba(0, 0, 0, 0.1), 0 4px 11px rgba(0, 0, 0, 0.1)',
    option: {
      textAlign: 'left',
      selectedColor: '#fff',
      selectedBgColor: '#007bff',
      padding: '0.375rem 0.75rem',
      focusedBgColor: 'rgba(0, 123, 255, 0.20)'
    }
  },
  noOptions: {
    fontSize: '1.25rem',
    margin: '0.25rem 0',
    color: 'hsl(0, 0%, 60%)',
    padding: '0.375rem 0.75rem'
  },
  multiValue: {
    margin: '1px 2px',
    borderRadius: '0.25rem',
    backgroundColor: '#e7edf3',
    animation: FADE_IN_ANIMATION_CSS,
    label: {
      fontSize: '0.825em',
      borderRadius: '0.25rem',
      padding: '1px 2px 1px 6px'
    },
    clear: {
      fontWeight: 700,
      padding: '0 7px',
      fontSize: '0.67em',
      alignItems: 'center',
      borderRadius: '0.25rem',
      transition: 'color 0.15s ease-in-out, background-color 0.15s ease-in-out'
    }
  }
});

var FilterMatchEnum;

(function (FilterMatchEnum) {
  FilterMatchEnum["ANY"] = "any";
  FilterMatchEnum["START"] = "start";
})(FilterMatchEnum || (FilterMatchEnum = {}));

const ValueIndexEnum = {
  NEXT: 0,
  PREVIOUS: 1
};
const OptionIndexEnum = {
  UP: 0,
  DOWN: 1,
  FIRST: 2,
  LAST: 3
};

const useDebounce = (value, delay) => {
  const [debouncedValue, setDebouncedValue] = useState(value);
  useEffect(() => {
    if (delay === undefined) {
      return;
    }

    const handler = setTimeout(() => {
      setDebouncedValue(value);
    }, delay);
    return () => {
      clearTimeout(handler);
    };
  }, [value, delay]);
  return delay === undefined ? value : debouncedValue;
};

const useUpdateEffect = (effect, deps) => {
  const isFirstRender = useRef(true);
  useEffect(() => {
    if (isFirstRender.current) {
      isFirstRender.current = false;
    } else {
      return effect();
    }
  }, deps);
};

const MENU_ITEM_SIZE_DEFAULT = 35;
const MENU_MAX_HEIGHT_DEFAULT = 300;
const FOCUSED_MULTI_DEFAULT = null;
const ON_CHANGE_SINGLE_VALUE_DEFAULT = null;
const LOADING_MSG_DEFAULT = 'Loading...';
const NO_OPTIONS_MSG_DEFAULT = 'No options';
const PLACEHOLDER_DEFAULT = 'Select option...';
const OPTIONS_DEFAULT = [];
const SELECTED_OPTION_DEFAULT = [];
const FOCUSED_OPTION_DEFAULT = {
  index: -1
};

const OVERFLOW_REGEXP = /(auto|scroll)/;
const IE_BROWSER_REGEXP = /MSIE|Trident/;
const DIACRITICS_REGEXP = /[\u0300-\u036f]/g;

const _attrPrefix = 'rfs';

const _isTest = process.env.NODE_ENV === 'test';

const IME_KEY_CODE = 229;
const MOUSE_DOWN_EVENT_TYPE = 'mousedown';
const OPTION_CLS = `${_attrPrefix}-option`;
const OPTION_FOCUSED_CLS = `${OPTION_CLS}-focused`;
const OPTION_SELECTED_CLS = `${OPTION_CLS}-selected`;
const OPTION_DISABLED_CLS = `${OPTION_CLS}-disabled`;
const CARET_ICON_CLS = `${_attrPrefix}-caret-icon`;
const CLEAR_ICON_CLS = `${_attrPrefix}-clear-icon`;
const LOADING_DOTS_CLS = `${_attrPrefix}-loading-dots`;
const AUTOSIZE_INPUT_CLS = `${_attrPrefix}-autosize-input`;
const MENU_CONTAINER_CLS = `${_attrPrefix}-menu-container`;
const SELECT_CONTAINER_CLS = `${_attrPrefix}-select-container`;
const CONTROL_CONTAINER_CLS = `${_attrPrefix}-control-container`;
const CLEAR_ICON_TESTID = _isTest ? CLEAR_ICON_CLS : undefined;
const CARET_ICON_TESTID = _isTest ? CARET_ICON_CLS : undefined;
const AUTOSIZE_INPUT_TESTID = _isTest ? AUTOSIZE_INPUT_CLS : undefined;
const MENU_CONTAINER_TESTID = _isTest ? MENU_CONTAINER_CLS : undefined;
const ARIA_LIVE_TESTID = _isTest ? `${_attrPrefix}-aria-live` : undefined;
const CLEAR_ICON_MV_TESTID = _isTest ? `${CLEAR_ICON_CLS}-mv` : undefined;
const SELECT_CONTAINER_TESTID = _isTest ? SELECT_CONTAINER_CLS : undefined;
const CONTROL_CONTAINER_TESTID = _isTest ? CONTROL_CONTAINER_CLS : undefined;

function stripDiacritics(value) {
  return value.normalize('NFD').replace(DIACRITICS_REGEXP, '');
}

function isDocumentElement(el) {
  return el === document.documentElement || el === document.body || el === window;
}

function easeOutCubic(t, s, c, d) {
  return c * ((t = t / d - 1) * t * t + 1) + s;
}

function getScrollTop(el) {
  return isDocumentElement(el) ? window.pageYOffset : el.scrollTop;
}

function scrollTo(el, top) {
  if (isDocumentElement(el)) {
    window.scrollTo(0, top);
  } else {
    el.scrollTop = top;
  }
}

function getScrollParent(el) {
  let style = getComputedStyle(el);
  const excludeStaticParent = style.position === 'absolute';

  if (style.position === 'fixed') {
    return document.documentElement;
  }

  for (let parent = el; parent = parent ? parent.parentElement : null;) {
    style = getComputedStyle(parent);

    if (excludeStaticParent && style.position === 'static') {
      continue;
    } else if (OVERFLOW_REGEXP.test(`${style.overflow}${style.overflowY}${style.overflowX}`)) {
      return parent;
    }
  }

  return document.documentElement;
}

function smoothScrollTo(element, to, duration = 300, callback) {
  let currentTime = 0;
  const start = getScrollTop(element);
  const change = to - start;

  function smoothScroller() {
    currentTime += 5;
    const top = easeOutCubic(currentTime, start, change, duration);
    scrollTo(element, top);

    if (currentTime < duration) {
      window.requestAnimationFrame(smoothScroller);
    } else {
      callback && callback();
    }
  }

  window.requestAnimationFrame(smoothScroller);
}

function isArrayWithLength(test) {
  return Array.isArray(test) && !!test.length;
}
function isPlainObject(test) {
  return test && typeof test === 'object' && !Array.isArray(test);
}
function isTouchDevice() {
  return window.matchMedia('(pointer: coarse)').matches;
}
function isBrowserIE() {
  return IE_BROWSER_REGEXP.test(window.navigator.userAgent);
}
function optionClassName(isDisabled, isSelected, isFocused) {
  let className = OPTION_CLS;
  if (isDisabled) className += ' ' + OPTION_DISABLED_CLS;
  if (isSelected) className += ' ' + OPTION_SELECTED_CLS;
  if (isFocused) className += ' ' + OPTION_FOCUSED_CLS;
  return className;
}
function trimAndFormatFilterStr(value, filterIgnoreCase, filterIgnoreAccents) {
  let trimVal = value.trim();

  if (filterIgnoreCase) {
    trimVal = trimVal.toLowerCase();
  }

  return !filterIgnoreAccents ? trimVal : stripDiacritics(trimVal);
}
function mergeDeep(target, source) {
  const output = Object.assign({}, target);
  Object.keys(source).forEach(key => {
    const sourceVal = source[key];

    if (isPlainObject(sourceVal) && key !== 'animation') {
      output[key] = key in target ? mergeDeep(target[key], sourceVal) : sourceVal;
    } else {
      output[key] = sourceVal || '';
    }
  });
  return output;
}
function scrollMenuIntoViewOnOpen(menuEl, menuScrollDuration, scrollMenuIntoView, handleOnMenuOpen) {
  if (!scrollMenuIntoView || !menuEl || !menuEl.getBoundingClientRect) {
    handleOnMenuOpen();
    return;
  }

  const {
    top: menuTop,
    bottom: menuBottom,
    height: menuHeight
  } = menuEl.getBoundingClientRect();
  const viewHeight = window.innerHeight;
  const viewSpaceBelow = viewHeight - menuTop;

  if (viewSpaceBelow >= menuHeight) {
    handleOnMenuOpen();
    return;
  }

  const scrollParent = getScrollParent(menuEl);
  const scrollTop = getScrollTop(scrollParent);
  const scrollSpaceBelow = scrollParent.getBoundingClientRect().height - scrollTop - menuTop;

  if (scrollSpaceBelow < menuHeight) {
    handleOnMenuOpen(scrollSpaceBelow);
    return;
  }

  const marginBottom = parseInt(getComputedStyle(menuEl).marginBottom || '0', 10);
  const scrollDown = menuBottom - viewHeight + scrollTop + marginBottom;
  smoothScrollTo(scrollParent, scrollDown, menuScrollDuration, handleOnMenuOpen);
}
function validateSetValueParam(values, menuOptions, getOptionValueCB) {
  if (values === null || values === undefined) {
    return SELECTED_OPTION_DEFAULT;
  }

  const validValues = normalizeValue(values).reduce((acc, x) => {
    isPlainObject(x) && acc.push(getOptionValueCB(x));
    return acc;
  }, []);

  if (!isArrayWithLength(validValues)) {
    return SELECTED_OPTION_DEFAULT;
  }

  const results = [];
  const validValuesUniq = [...new Set(validValues)];

  for (const option of menuOptions) {
    if (validValuesUniq.includes(getOptionValueCB(option))) {
      results.push(option);

      if (validValuesUniq.length === results.length) {
        break;
      }
    }
  }

  return results;
}
function normalizeValue(value, getOptionValueCB, getOptionLabelCB) {
  const initialValues = Array.isArray(value) ? value.filter(Boolean) : isPlainObject(value) ? [value] : SELECTED_OPTION_DEFAULT;

  if (!getOptionValueCB || !getOptionLabelCB || !isArrayWithLength(initialValues)) {
    return initialValues;
  }

  return initialValues.map(initVal => ({
    data: initVal,
    value: getOptionValueCB(initVal),
    label: getOptionLabelCB(initVal)
  }));
}

const useMenuHeight = (menuRef, menuOpen, menuHeightDefault, menuScrollDuration, scrollMenuIntoView, onMenuOpen, onMenuClose) => {
  const resetMenuHeight = useRef(false);
  const [menuHeight, setMenuHeight] = useState(menuHeightDefault);
  useUpdateEffect(() => {
    if (menuOpen) {
      const handleOnMenuOpen = availableSpace => {
        onMenuOpen && onMenuOpen();

        if (availableSpace) {
          resetMenuHeight.current = true;
          setMenuHeight(availableSpace);
        }
      };

      scrollMenuIntoViewOnOpen(menuRef.current, menuScrollDuration, scrollMenuIntoView, handleOnMenuOpen);
    } else {
      onMenuClose && onMenuClose();

      if (resetMenuHeight.current) {
        resetMenuHeight.current = false;
        setMenuHeight(menuHeightDefault);
      }
    }
  }, [menuRef, menuOpen, onMenuClose, onMenuOpen, menuHeightDefault, scrollMenuIntoView, menuScrollDuration]);
  return menuHeight;
};

const useMenuOptions = (options, debouncedInputValue, filterMatchFrom, selectedOption, getOptionValueCB, getOptionLabelCB, getIsOptionDisabled, getFilterOptionString, filterIgnoreCase, filterIgnoreAccents, isMulti, hideSelectedOptions, async) => {
  const [menuOptions, setMenuOptions] = useState(OPTIONS_DEFAULT);
  const searchValue = !async ? debouncedInputValue : '';
  const hideSelectedOptionsOrDefault = typeof hideSelectedOptions !== 'boolean' ? !!isMulti : hideSelectedOptions;
  useEffect(() => {
    const matchAny = filterMatchFrom === FilterMatchEnum.ANY;
    const normalizedInput = trimAndFormatFilterStr(searchValue, filterIgnoreCase, filterIgnoreAccents);
    const selectedValues = selectedOption.length ? selectedOption.map(({
      value
    }) => value) : undefined;

    const getIsOptionDisabledOrDefault = getIsOptionDisabled || (data => !!data.isDisabled);

    const getFilterOptionStringOrDefault = getFilterOptionString || (option => typeof option.label === 'string' ? option.label : `${option.label}`);

    const isOptionFilterMatch = menuOption => {
      const optionStr = getFilterOptionStringOrDefault(menuOption);
      const normalizedOptionLabel = trimAndFormatFilterStr(optionStr, filterIgnoreCase, filterIgnoreAccents);
      return matchAny ? normalizedOptionLabel.indexOf(normalizedInput) > -1 : normalizedOptionLabel.substr(0, normalizedInput.length) === normalizedInput;
    };

    const parseMenuOption = data => {
      const value = getOptionValueCB(data);
      const menuOption = Object.assign({
        data,
        value,
        label: getOptionLabelCB(data)
      }, getIsOptionDisabledOrDefault(data) && {
        isDisabled: true
      }, {}, selectedValues && selectedValues.includes(value) && {
        isSelected: true
      });

      if (normalizedInput && !isOptionFilterMatch(menuOption) || hideSelectedOptionsOrDefault && menuOption.isSelected) {
        return;
      }

      return menuOption;
    };

    const menuOptionsOrDefault = options.reduce((acc, data) => {
      const option = parseMenuOption(data);
      option && acc.push(option);
      return acc;
    }, []);
    setMenuOptions(menuOptionsOrDefault || OPTIONS_DEFAULT);
  }, [options, selectedOption, searchValue, hideSelectedOptionsOrDefault, filterMatchFrom, filterIgnoreCase, filterIgnoreAccents, getFilterOptionString, getIsOptionDisabled, getOptionValueCB, getOptionLabelCB]);
  return menuOptions;
};

const Option = React.memo(({
  index,
  style,
  data: {
    menuOptions,
    selectOption,
    renderOptionLabel,
    focusedOptionIndex
  }
}) => {
  const {
    data,
    value,
    label,
    isDisabled,
    isSelected
  } = menuOptions[index];
  const className = optionClassName(isDisabled, isSelected, index === focusedOptionIndex);
  const onClickHandler = !isDisabled ? () => selectOption({
    data,
    value,
    label
  }, isSelected) : undefined;
  return React.createElement("div", {
    style: style,
    className: className,
    onClick: onClickHandler
  }, renderOptionLabel(data));
}, areEqual);
Option.displayName = 'Option';

const NoOptionsMsg = styled.div`
  text-align: center;
  ${({
  theme: {
    noOptions
  }
}) => `
    color: ${noOptions.color};
    margin: ${noOptions.margin};
    padding: ${noOptions.padding};
    font-size: ${noOptions.fontSize};
    ${noOptions.css || ''}
  `}
`;
const Menu = React.forwardRef(({
  width,
  itemSize,
  maxHeight,
  isLoading,
  loadingMsg,
  menuOptions,
  selectOption,
  noOptionsMsg,
  overscanCount,
  renderOptionLabel,
  focusedOptionIndex
}, ref) => {
  const itemData = useMemo(() => ({
    menuOptions,
    selectOption,
    renderOptionLabel,
    focusedOptionIndex
  }), [menuOptions, focusedOptionIndex, selectOption, renderOptionLabel]);

  if (isLoading) {
    return React.createElement(NoOptionsMsg, null, loadingMsg);
  }

  return React.createElement(Fragment, null, React.createElement(FixedSizeList, {
    ref: ref,
    width: width,
    itemSize: itemSize,
    itemData: itemData,
    overscanCount: overscanCount,
    itemCount: menuOptions.length,
    height: Math.min(maxHeight, menuOptions.length * itemSize)
  }, Option), !isArrayWithLength(menuOptions) && React.createElement(NoOptionsMsg, null, noOptionsMsg));
});
Menu.displayName = 'Menu';

const MultiValueWrapper = styled.div`
  min-width: 0;
  display: flex;
  animation: ${({
  theme
}) => css`${theme.multiValue.animation}`};
  ${({
  theme: {
    multiValue
  }
}) => `
    margin: ${multiValue.margin};
    border-radius: ${multiValue.borderRadius};
    background-color: ${multiValue.backgroundColor};
    ${multiValue.css || ''}
  `}
`;
const Label = styled.div`
  overflow: hidden;
  white-space: nowrap;
  text-overflow: ellipsis;
  ${({
  theme: {
    multiValue: {
      label
    }
  }
}) => `
    padding: ${label.padding};
    font-size: ${label.fontSize};
    border-radius: ${label.borderRadius};
  `}
`;
const Clear = styled.div`
  display: flex;
  ${({
  isFocused,
  theme: {
    color,
    multiValue: {
      clear
    }
  }
}) => `
    padding: ${clear.padding};
    font-size: ${clear.fontSize};
    transition: ${clear.transition};
    align-items: ${clear.alignItems};
    font-weight: ${clear.fontWeight};
    border-radius: ${clear.borderRadius};
    background-color: ${isFocused ? color.dangerLight : 'transparent'};
    :hover {
      color: ${color.danger};
      background-color: ${color.dangerLight};
    }
  `}
`;

const MultiValue = ({
  data,
  value,
  isFocused,
  renderOptionLabel,
  removeSelectedOption
}) => React.createElement(MultiValueWrapper, null, React.createElement(Label, null, renderOptionLabel(data)), React.createElement(Clear, {
  "aria-hidden": 'true',
  isFocused: isFocused,
  "data-testid": CLEAR_ICON_MV_TESTID,
  onTouchEnd: e => removeSelectedOption(value, e),
  onMouseDown: e => removeSelectedOption(value, e)
}, "X"));

const SingleValue = styled.div`
  top: 50%;
  overflow: hidden;
  position: absolute;
  white-space: nowrap;
  box-sizing: border-box;
  text-overflow: ellipsis;
  transform: translateY(-50%);
  max-width: calc(100% - 0.5rem);
`;
const Placeholder = styled.div`
  top: 50%;
  overflow: hidden;
  position: absolute;
  white-space: nowrap;
  box-sizing: border-box;
  text-overflow: ellipsis;
  transform: translateY(-50%);
  color: ${({
  theme
}) => theme.color.placeholder};
`;
const Value = React.memo(({
  isMulti,
  inputValue,
  placeholder,
  selectedOption,
  focusedMultiValue,
  renderOptionLabel,
  removeSelectedOption
}) => {
  if (inputValue && (!isMulti || isMulti && !isArrayWithLength(selectedOption))) {
    return null;
  }

  if (!isArrayWithLength(selectedOption)) {
    return React.createElement(Placeholder, null, placeholder);
  }

  if (!isMulti) {
    const optionLabel = renderOptionLabel(selectedOption[0].data);
    return React.createElement(SingleValue, null, optionLabel);
  }

  return React.createElement(Fragment, null, selectedOption.map(({
    data,
    value
  }) => React.createElement(MultiValue, {
    key: value,
    data: data,
    value: value,
    renderOptionLabel: renderOptionLabel,
    isFocused: value === focusedMultiValue,
    removeSelectedOption: removeSelectedOption
  })));
});
Value.displayName = 'Value';

const INPUT_MIN_WIDTH_PX = 2;
const STATIC_ATTRIBUTES = {
  type: 'text',
  spellCheck: false,
  autoCorrect: 'off',
  autoComplete: 'off',
  autoCapitalize: 'none',
  'aria-autocomplete': 'list',
  'data-testid': AUTOSIZE_INPUT_TESTID
};
const SizerDiv = styled.div`
  top: 0;
  left: 0;
  height: 0;
  overflow: scroll;
  white-space: pre;
  position: absolute;
  visibility: hidden;
  font-size: inherit;
  font-weight: inherit;
  font-family: inherit;
  ${({
  theme
}) => theme.input.css}
`;
const Input = styled.input`
  border: 0;
  outline: 0;
  padding: 0;
  cursor: text;
  background: 0;
  color: inherit;
  font-size: inherit;
  font-weight: inherit;
  font-family: inherit;
  box-sizing: content-box;

  :read-only {
    opacity: 0;
    cursor: default;
  }

  ${({
  theme
}) => theme.input.css}
  ${() => isBrowserIE() && '::-ms-clear { display: none; }'}
`;
const AutosizeInput = React.memo(React.forwardRef(({
  id,
  onBlur,
  onFocus,
  readOnly,
  onChange,
  ariaLabel,
  inputValue,
  addClassNames,
  ariaLabelledBy
}, ref) => {
  const sizerRef = useRef(null);
  const [inputWidth, setInputWidth] = useState(INPUT_MIN_WIDTH_PX);
  const autosizeInputAttrs = Object.assign({}, STATIC_ATTRIBUTES, {
    'aria-label': ariaLabel,
    'aria-labelledby': ariaLabelledBy
  });
  useEffect(() => {
    if (sizerRef.current) {
      setInputWidth(sizerRef.current.scrollWidth + INPUT_MIN_WIDTH_PX);
    }
  }, [inputValue]);
  return React.createElement(Fragment, null, React.createElement(Input, Object.assign({
    id: id,
    ref: ref,
    onBlur: onBlur,
    onFocus: onFocus,
    value: inputValue,
    readOnly: readOnly
  }, autosizeInputAttrs, {
    style: {
      width: inputWidth
    },
    onChange: !readOnly ? onChange : undefined,
    className: addClassNames ? AUTOSIZE_INPUT_CLS : undefined
  })), React.createElement(SizerDiv, {
    ref: sizerRef
  }, inputValue));
}));
AutosizeInput.displayName = 'AutosizeInput';

const StyledLoadingDots = styled.div`
  display: flex;
  align-self: center;
  text-align: center;
  margin-right: 0.25rem;
  padding: ${({
  theme
}) => theme.loader.padding};

  > div {
    border-radius: 100%;
    display: inline-block;
    animation: ${({
  theme
}) => css`${theme.loader.animation}`};
    ${({
  theme: {
    loader
  }
}) => `
      width: ${loader.size};
      height: ${loader.size};
      background-color: ${loader.color};
    `}

    :nth-of-type(1) {
      animation-delay: -0.272s;
    }

    :nth-of-type(2) {
      animation-delay: -0.136s;
    }
  }
`;

const LoadingDots = ({
  addClassNames
}) => React.createElement(StyledLoadingDots, {
  "aria-hidden": 'true',
  className: addClassNames ? LOADING_DOTS_CLS : undefined
}, React.createElement("div", null), React.createElement("div", null), React.createElement("div", null));

const IndicatorIconsWrapper = styled.div`
  display: flex;
  flex-shrink: 0;
  align-items: center;
  align-self: stretch;
  box-sizing: border-box;
`;
const IndicatorIcon = styled.div`
  display: flex;
  box-sizing: border-box;
  ${({
  theme: {
    icon
  }
}) => `
    color: ${icon.color};
    padding: ${icon.padding};
    :hover {
      color: ${icon.hoverColor};
    }
  `}
  ${({
  theme
}) => theme.icon.css}
`;
const Clear$1 = styled.div`
  overflow: hidden;
  animation: ${({
  theme
}) => css`${theme.icon.clear.animation}`};
  ${({
  theme: {
    icon: {
      clear
    }
  }
}) => `
    font-size: ${clear.fontSize};
    transition: ${clear.transition};
    font-weight: ${clear.fontWeight};
    font-family: ${clear.fontFamily};
  `}
`;
const Caret = styled.div`
  ${({
  theme: {
    icon: {
      caret
    }
  }
}) => `
    transition: ${caret.transition};
    border-top: ${caret.size} dashed;
    border-left: ${caret.size} solid transparent;
    border-right: ${caret.size} solid transparent;
  `}

  ${({
  menuOpen,
  isInvalid,
  theme: {
    color
  }
}) => menuOpen && `
      transform: rotate(180deg);
      color: ${isInvalid ? color.danger : color.caretActive || color.primary};
    `}
`;
const Separator = styled.div`
  width: 1px;
  margin: 0.5rem 0;
  align-self: stretch;
  box-sizing: border-box;
  background-color: ${({
  theme
}) => theme.color.iconSeparator || theme.color.border};
`;
const IndicatorIcons = React.memo(({
  menuOpen,
  clearIcon,
  caretIcon,
  isInvalid,
  showClear,
  isLoading,
  loadingNode,
  addClassNames,
  onCaretMouseDown,
  onClearMouseDown
}) => React.createElement(IndicatorIconsWrapper, null, showClear && !isLoading && React.createElement(IndicatorIcon, {
  "aria-hidden": 'true',
  onTouchEnd: onClearMouseDown,
  onMouseDown: onClearMouseDown,
  "data-testid": CLEAR_ICON_TESTID
}, clearIcon || React.createElement(Clear$1, {
  className: addClassNames ? CLEAR_ICON_CLS : undefined
}, "X")), isLoading && (loadingNode || React.createElement(LoadingDots, {
  addClassNames: addClassNames
})), React.createElement(Separator, null), React.createElement(IndicatorIcon, {
  "aria-hidden": 'true',
  onTouchEnd: onCaretMouseDown,
  onMouseDown: onCaretMouseDown,
  "data-testid": CARET_ICON_TESTID
}, caretIcon || React.createElement(Caret, {
  menuOpen: menuOpen,
  isInvalid: isInvalid,
  className: addClassNames ? CARET_ICON_CLS : undefined
}))));
IndicatorIcons.displayName = 'IndicatorIcons';

const A11yText = styled.span`
  border: 0;
  padding: 0;
  width: 1px;
  height: 1px;
  margin: -1px;
  overflow: hidden;
  position: absolute;
  white-space: nowrap;
  clip: rect(0 0 0 0);
  clip-path: inset(50%);
`;

const AriaLiveRegion = ({
  menuOpen,
  isFocused,
  ariaLabel,
  inputValue,
  optionCount,
  isSearchable,
  focusedOption,
  selectedOption
}) => {
  if (!isFocused) {
    return null;
  }

  const {
    value: focusedOptionValue,
    label: focusedOptionLabel,
    index: focusedOptionIndex,
    isDisabled: isFocusedOptionDisabled
  } = focusedOption;
  const selectedOptionLabel = isArrayWithLength(selectedOption) ? selectedOption.map(({
    label
  }) => label).join(' ') : 'N/A';
  const selectedOptionMsg = `Selected option: ${selectedOptionLabel}`;
  const optionsMsg = `${optionCount} result(s) available${inputValue ? ` for search input ${inputValue}` : ''}.`;
  const focusedMsg = focusedOptionValue ? `Focused option: ${focusedOptionLabel}${isFocusedOptionDisabled ? ' - disabled' : ''}, ${focusedOptionIndex + 1} of ${optionCount}.` : '';
  const menuMsg = menuOpen ? 'Use Up and Down arrow keys to choose options, press Enter or Tab to select the currently focused option, press Escape to close the menu.' : `${ariaLabel || 'Select'} is focused${isSearchable ? ', type to filter options' : ''}, press Down arrow key to open the menu.`;
  return React.createElement(A11yText, {
    "aria-live": 'polite',
    "data-testid": ARIA_LIVE_TESTID
  }, React.createElement("p", null, selectedOptionMsg), React.createElement("p", null, `${focusedMsg} ${optionsMsg} ${menuMsg}`));
};

const SelectWrapper = styled.div`
  position: relative;
  box-sizing: border-box;
  ${({
  theme
}) => theme.select.css}
`;
const ValueWrapper = styled.div`
  flex: 1 1 0%;
  display: flex;
  flex-wrap: wrap;
  overflow: hidden;
  position: relative;
  align-items: center;
  box-sizing: border-box;
  padding: ${({
  theme
}) => theme.control.padding};
`;
const ControlWrapper = styled.div`
  outline: 0;
  display: flex;
  flex-wrap: wrap;
  cursor: default;
  position: relative;
  align-items: center;
  box-sizing: border-box;
  justify-content: space-between;
  ${({
  isDisabled,
  isFocused,
  isInvalid,
  theme: {
    control,
    color
  }
}) => `
    min-height: ${control.minHeight};
    transition: ${control.transition};
    border-style: ${control.borderStyle};
    border-width: ${control.borderWidth};
    border-radius: ${control.borderRadius};
    border-color: ${isInvalid ? color.danger : isFocused ? control.focusedBorderColor : color.border};
    ${isDisabled ? 'pointer-events: none;' : ''}
    ${control.height ? `height: ${control.height};` : ''}
    ${control.backgroundColor || isDisabled ? `background-color: ${isDisabled ? color.disabled : control.backgroundColor};` : ''}
    ${isFocused ? `box-shadow: ${control.boxShadow} ${isInvalid ? color.dangerLight : control.boxShadowColor};` : ''}
    ${control.css || ''}
  `}
`;
const MenuWrapper = styled.div`
  z-index: 999;
  cursor: default;
  position: absolute;
  ${({
  hideMenu,
  theme: {
    menu
  }
}) => `
    width: ${menu.width};
    margin: ${menu.margin};
    padding: ${menu.padding};
    box-shadow: ${menu.boxShadow};
    border-radius: ${menu.borderRadius};
    background-color: ${menu.backgroundColor};
    ${hideMenu ? 'display: none;' : ''}
  `}

  animation: ${({
  theme
}) => css`${theme.menu.animation}`};
  ${({
  theme
}) => theme.menu.css}

  .${OPTION_CLS} {
    display: block;
    overflow: hidden;
    user-select: none;
    white-space: nowrap;
    text-overflow: ellipsis;
    -webkit-tap-highlight-color: transparent;
    ${({
  theme: {
    menu: {
      option
    }
  }
}) => `
      padding: ${option.padding};
      text-align: ${option.textAlign};

      &.${OPTION_FOCUSED_CLS},
      &:hover:not(.${OPTION_DISABLED_CLS}):not(.${OPTION_SELECTED_CLS}) {
        background-color: ${option.focusedBgColor};
      }

      &.${OPTION_SELECTED_CLS} {
        color: ${option.selectedColor};
        background-color: ${option.selectedBgColor};
      }

      &.${OPTION_DISABLED_CLS} {
        opacity: 0.35;
      }
    `}
  }
`;
const Select = React.forwardRef(({
  async,
  isMulti,
  inputId,
  selectId,
  autoFocus,
  isLoading,
  onKeyDown,
  clearIcon,
  caretIcon,
  isInvalid,
  ariaLabel,
  menuWidth,
  isDisabled,
  inputDelay,
  onMenuOpen,
  onMenuClose,
  onInputBlur,
  isClearable,
  themeConfig,
  loadingNode,
  onInputFocus,
  initialValue,
  onInputChange,
  addClassNames,
  ariaLabelledBy,
  onOptionChange,
  onSearchChange,
  getOptionLabel,
  getOptionValue,
  openMenuOnFocus,
  isAriaLiveEnabled,
  menuOverscanCount,
  blurInputOnSelect,
  renderOptionLabel,
  menuScrollDuration,
  filterIgnoreAccents,
  hideSelectedOptions,
  getIsOptionDisabled,
  getFilterOptionString,
  isSearchable = true,
  openMenuOnClick = true,
  filterIgnoreCase = true,
  tabSelectsOption = true,
  closeMenuOnSelect = true,
  scrollMenuIntoView = true,
  backspaceClearsValue = true,
  options = OPTIONS_DEFAULT,
  loadingMsg = LOADING_MSG_DEFAULT,
  placeholder = PLACEHOLDER_DEFAULT,
  filterMatchFrom = FilterMatchEnum.ANY,
  noOptionsMsg = NO_OPTIONS_MSG_DEFAULT,
  menuItemSize = MENU_ITEM_SIZE_DEFAULT,
  menuMaxHeight = MENU_MAX_HEIGHT_DEFAULT
}, ref) => {
  const prevMenuOptionsLength = useRef();
  const onChangeEventValue = useRef(false);
  const listRef = useRef(null);
  const menuRef = useRef(null);
  const inputRef = useRef(null);
  const [inputValue, setInputValue] = useState('');
  const [menuOpen, setMenuOpen] = useState(false);
  const [isFocused, setIsFocused] = useState(false);
  const [focusedOption, setFocusedOption] = useState(FOCUSED_OPTION_DEFAULT);
  const [focusedMultiValue, setFocusedMultiValue] = useState(FOCUSED_MULTI_DEFAULT);
  const theme = useMemo(() => {
    return isPlainObject(themeConfig) ? mergeDeep(DefaultThemeObj, themeConfig) : DefaultThemeObj;
  }, [themeConfig]);
  const getOptionLabelCB = useMemo(() => {
    return getOptionLabel || (data => data.label);
  }, [getOptionLabel]);
  const getOptionValueCB = useMemo(() => {
    return getOptionValue || (data => data.value);
  }, [getOptionValue]);
  const renderOptionLabelCB = useMemo(() => {
    return renderOptionLabel || getOptionLabelCB;
  }, [renderOptionLabel, getOptionLabelCB]);
  const debouncedInputValue = useDebounce(inputValue, inputDelay);
  const [selectedOption, setSelectedOption] = useState(() => normalizeValue(initialValue, getOptionValueCB, getOptionLabelCB));
  const menuHeight = useMenuHeight(menuRef, menuOpen, menuMaxHeight, menuScrollDuration, scrollMenuIntoView, onMenuOpen, onMenuClose);
  const menuOptions = useMenuOptions(options, debouncedInputValue, filterMatchFrom, selectedOption, getOptionValueCB, getOptionLabelCB, getIsOptionDisabled, getFilterOptionString, filterIgnoreCase, filterIgnoreAccents, isMulti, hideSelectedOptions, async);

  const blurInput = () => {
    inputRef.current.blur();
  };

  const focusInput = () => {
    inputRef.current.focus();
  };

  const scrollToItemIndex = index => {
    if (listRef.current) {
      listRef.current.scrollToItem(index);
    }
  };

  const removeSelectedOption = useCallback((value, e) => {
    if (e) {
      e.stopPropagation();
      e.type === MOUSE_DOWN_EVENT_TYPE && e.preventDefault();
    }

    setSelectedOption(prevSelectedOption => prevSelectedOption.filter(x => x.value !== value));
  }, []);
  const openMenuAndFocusOption = useCallback(position => {
    if (!isArrayWithLength(menuOptions)) {
      setMenuOpen(true);
      return;
    }

    const selectedIndex = !isMulti ? menuOptions.findIndex(({
      isSelected
    }) => isSelected) : -1;
    const index = selectedIndex > -1 ? selectedIndex : position === OptionIndexEnum.FIRST ? 0 : menuOptions.length - 1;
    setMenuOpen(true);
    setFocusedOption(Object.assign({
      index
    }, menuOptions[index]));
    scrollToItemIndex(index);
  }, [isMulti, menuOptions]);
  const selectOption = useCallback((option, isSelected) => {
    if (isSelected) {
      isMulti && removeSelectedOption(option.value);
    } else {
      setSelectedOption(prevSelectedOption => !isMulti ? [option] : [...prevSelectedOption, option]);
    }

    const blurInputOnSelectOrDefault = typeof blurInputOnSelect === 'boolean' ? blurInputOnSelect : isTouchDevice();

    if (blurInputOnSelectOrDefault) {
      blurInput();
    } else if (closeMenuOnSelect) {
      setMenuOpen(false);
      setInputValue('');
    }
  }, [isMulti, closeMenuOnSelect, blurInputOnSelect, removeSelectedOption]);
  useImperativeHandle(ref, () => ({
    blur: blurInput,
    focus: focusInput,
    clearValue: () => {
      setSelectedOption(SELECTED_OPTION_DEFAULT);
      setFocusedOption(FOCUSED_OPTION_DEFAULT);
    },
    setValue: option => {
      const validatedSelectedOption = validateSetValueParam(option, menuOptions, getOptionValueCB);
      setSelectedOption(validatedSelectedOption);
    },
    toggleMenu: state => {
      if (state === true || state === undefined && !menuOpen) {
        !isFocused && focusInput();
        openMenuAndFocusOption(OptionIndexEnum.FIRST);
      } else {
        blurInput();
      }
    }
  }));
  useEffect(() => {
    autoFocus && focusInput();
  }, [autoFocus]);
  useEffect(() => {
    if (isFocused && openMenuOnFocus) {
      openMenuAndFocusOption(OptionIndexEnum.FIRST);
    }
  }, [isFocused, openMenuOnFocus, openMenuAndFocusOption]);
  useEffect(() => {
    if (onSearchChange && onChangeEventValue.current) {
      onChangeEventValue.current = false;
      onSearchChange(debouncedInputValue);
    }
  }, [onSearchChange, debouncedInputValue]);
  useUpdateEffect(() => {
    if (onOptionChange) {
      const normalizedOptionValue = isMulti ? selectedOption.map(({
        data
      }) => data) : isArrayWithLength(selectedOption) ? selectedOption[0].data : ON_CHANGE_SINGLE_VALUE_DEFAULT;
      onOptionChange(normalizedOptionValue);
    }
  }, [isMulti, selectedOption, onOptionChange]);
  useUpdateEffect(() => {
    const curLength = menuOptions.length;
    const inputChanged = curLength > 0 && (async || curLength !== options.length || prevMenuOptionsLength.current === 0);

    if (curLength === 0) {
      setFocusedOption(FOCUSED_OPTION_DEFAULT);
    } else if (curLength === 1 || inputChanged) {
      setFocusedOption(Object.assign({
        index: 0
      }, menuOptions[0]));
      scrollToItemIndex(0);
    }

    prevMenuOptionsLength.current = curLength;
  }, [async, options, menuOptions]);

  const selectOptionFromFocused = () => {
    const {
      data,
      value,
      label,
      isSelected,
      isDisabled: focusedOptionDisabled
    } = focusedOption;

    if (data && !focusedOptionDisabled) {
      selectOption({
        data,
        value,
        label
      }, isSelected);
    }
  };

  const focusValueOnArrowKey = direction => {
    if (!isArrayWithLength(selectedOption)) {
      return;
    }

    const curFocusedIndex = focusedMultiValue ? selectedOption.findIndex(({
      value
    }) => value === focusedMultiValue) : -1;
    let nextFocusedIndex = -1;
    const lastValuesIndex = selectedOption.length - 1;

    if (direction === ValueIndexEnum.NEXT) {
      nextFocusedIndex = curFocusedIndex > -1 && curFocusedIndex < lastValuesIndex ? curFocusedIndex + 1 : -1;
    } else {
      nextFocusedIndex = curFocusedIndex !== 0 ? curFocusedIndex === -1 ? lastValuesIndex : curFocusedIndex - 1 : 0;
    }

    const nextFocusedVal = nextFocusedIndex === -1 ? FOCUSED_MULTI_DEFAULT : selectedOption[nextFocusedIndex].value;
    if (focusedOption.data) setFocusedOption(FOCUSED_OPTION_DEFAULT);
    if (nextFocusedVal !== focusedMultiValue) setFocusedMultiValue(nextFocusedVal);
  };

  const focusOptionOnArrowKey = direction => {
    if (!isArrayWithLength(menuOptions)) {
      return;
    }

    const index = direction === OptionIndexEnum.DOWN ? (focusedOption.index + 1) % menuOptions.length : focusedOption.index > 0 ? focusedOption.index - 1 : menuOptions.length - 1;
    focusedMultiValue && setFocusedMultiValue(FOCUSED_MULTI_DEFAULT);
    setFocusedOption(Object.assign({
      index
    }, menuOptions[index]));
    scrollToItemIndex(index);
  };

  const handleOnKeyDown = e => {
    if (isDisabled) {
      return;
    }

    if (onKeyDown) {
      onKeyDown(e);
      if (e.defaultPrevented) return;
    }

    switch (e.key) {
      case 'ArrowDown':
        if (menuOpen) {
          focusOptionOnArrowKey(OptionIndexEnum.DOWN);
        } else {
          openMenuAndFocusOption(OptionIndexEnum.FIRST);
        }

        break;

      case 'ArrowUp':
        if (menuOpen) {
          focusOptionOnArrowKey(OptionIndexEnum.UP);
        } else {
          openMenuAndFocusOption(OptionIndexEnum.LAST);
        }

        break;

      case 'ArrowLeft':
        if (!isMulti || inputValue) {
          return;
        }

        focusValueOnArrowKey(ValueIndexEnum.PREVIOUS);
        break;

      case 'ArrowRight':
        if (!isMulti || inputValue) {
          return;
        }

        focusValueOnArrowKey(ValueIndexEnum.NEXT);
        break;

      case ' ':
        if (inputValue) {
          return;
        } else if (!menuOpen) {
          openMenuAndFocusOption(OptionIndexEnum.FIRST);
          break;
        } else if (!focusedOption.data) {
          return;
        }

        selectOptionFromFocused();
        break;

      case 'Enter':
        if (menuOpen && e.keyCode !== IME_KEY_CODE) {
          selectOptionFromFocused();
        }

        break;

      case 'Escape':
        if (menuOpen) {
          setMenuOpen(false);
          setInputValue('');
        }

        break;

      case 'Tab':
        if (!menuOpen || !tabSelectsOption || !focusedOption.data || e.shiftKey) {
          return;
        }

        selectOptionFromFocused();
        break;

      case 'Delete':
      case 'Backspace':
        if (inputValue) {
          return;
        }

        if (focusedMultiValue) {
          const clearFocusedIndex = selectedOption.findIndex(({
            value
          }) => value === focusedMultiValue);
          const nexFocusedMultiValue = clearFocusedIndex > -1 && clearFocusedIndex < selectedOption.length - 1 ? selectedOption[clearFocusedIndex + 1].value : FOCUSED_MULTI_DEFAULT;
          removeSelectedOption(focusedMultiValue);
          setFocusedMultiValue(nexFocusedMultiValue);
        } else {
          if (!backspaceClearsValue) {
            return;
          }

          if (isArrayWithLength(selectedOption)) {
            if (isMulti) {
              const {
                value
              } = selectedOption[selectedOption.length - 1];
              removeSelectedOption(value);
            } else if (isClearable) {
              setSelectedOption(SELECTED_OPTION_DEFAULT);
            }
          }
        }

        break;

      default:
        return;
    }

    e.preventDefault();
  };

  const handleOnControlMouseDown = e => {
    if (isDisabled) return;
    if (!isFocused) focusInput();
    const tagNameNotInput = e.currentTarget.tagName !== 'INPUT';

    if (!menuOpen) {
      openMenuOnClick && openMenuAndFocusOption(OptionIndexEnum.FIRST);
    } else if (tagNameNotInput) {
      setMenuOpen(false);
      inputValue && setInputValue('');
    }

    if (tagNameNotInput) {
      e.preventDefault();
    }
  };

  const handleOnMenuMouseDown = e => {
    e.stopPropagation();
    e.preventDefault();
    focusInput();
  };

  const handleOnInputBlur = useCallback(e => {
    onInputBlur && onInputBlur(e);
    setIsFocused(false);
    setMenuOpen(false);
    setInputValue('');
  }, [onInputBlur]);
  const handleOnInputFocus = useCallback(e => {
    onInputFocus && onInputFocus(e);
    setIsFocused(true);
  }, [onInputFocus]);
  const handleOnInputChange = useCallback(e => {
    onChangeEventValue.current = true;
    onInputChange && onInputChange(e.currentTarget.value || '');
    setMenuOpen(true);
    setInputValue(e.currentTarget.value || '');
  }, [onInputChange]);
  const handleOnClearMouseDown = useCallback(e => {
    e.stopPropagation();
    e.type === MOUSE_DOWN_EVENT_TYPE && e.preventDefault();
    setSelectedOption(SELECTED_OPTION_DEFAULT);
    focusInput();
  }, []);
  const handleOnCaretMouseDown = useCallback(e => {
    e.stopPropagation();
    e.type === MOUSE_DOWN_EVENT_TYPE && e.preventDefault();
    focusInput();

    if (menuOpen) {
      setMenuOpen(false);
    } else {
      openMenuAndFocusOption(OptionIndexEnum.FIRST);
    }
  }, [menuOpen, openMenuAndFocusOption]);
  return React.createElement(ThemeProvider, {
    theme: theme
  }, React.createElement(SelectWrapper, {
    id: selectId,
    role: 'combobox',
    "aria-haspopup": 'listbox',
    "aria-controls": inputId,
    onKeyDown: handleOnKeyDown,
    "data-testid": SELECT_CONTAINER_TESTID,
    "aria-expanded": menuOpen ? 'true' : 'false',
    className: addClassNames ? SELECT_CONTAINER_CLS : undefined
  }, React.createElement(ControlWrapper, {
    isInvalid: isInvalid,
    isFocused: isFocused,
    isDisabled: isDisabled,
    onTouchEnd: handleOnControlMouseDown,
    onMouseDown: handleOnControlMouseDown,
    "data-testid": CONTROL_CONTAINER_TESTID,
    className: addClassNames ? CONTROL_CONTAINER_CLS : undefined
  }, React.createElement(ValueWrapper, null, React.createElement(Value, {
    isMulti: isMulti,
    inputValue: inputValue,
    placeholder: placeholder,
    selectedOption: selectedOption,
    focusedMultiValue: focusedMultiValue,
    renderOptionLabel: renderOptionLabelCB,
    removeSelectedOption: removeSelectedOption
  }), React.createElement(AutosizeInput, {
    id: inputId,
    ref: inputRef,
    ariaLabel: ariaLabel,
    inputValue: inputValue,
    onBlur: handleOnInputBlur,
    onFocus: handleOnInputFocus,
    addClassNames: addClassNames,
    onChange: handleOnInputChange,
    ariaLabelledBy: ariaLabelledBy,
    readOnly: isDisabled || !isSearchable || !!focusedMultiValue
  })), React.createElement(IndicatorIcons, {
    menuOpen: menuOpen,
    clearIcon: clearIcon,
    caretIcon: caretIcon,
    isInvalid: isInvalid,
    isLoading: isLoading,
    loadingNode: loadingNode,
    addClassNames: addClassNames,
    onClearMouseDown: handleOnClearMouseDown,
    showClear: !!(isClearable && !isDisabled && isArrayWithLength(selectedOption)),
    onCaretMouseDown: !isDisabled && !openMenuOnClick ? handleOnCaretMouseDown : undefined
  })), React.createElement(MenuWrapper, {
    ref: menuRef,
    hideMenu: !menuOpen,
    onMouseDown: handleOnMenuMouseDown,
    "data-testid": MENU_CONTAINER_TESTID,
    className: addClassNames ? MENU_CONTAINER_CLS : undefined
  }, React.createElement(Menu, {
    ref: listRef,
    isLoading: isLoading,
    maxHeight: menuHeight,
    itemSize: menuItemSize,
    loadingMsg: loadingMsg,
    menuOptions: menuOptions,
    noOptionsMsg: noOptionsMsg,
    selectOption: selectOption,
    overscanCount: menuOverscanCount,
    width: menuWidth || theme.menu.width,
    renderOptionLabel: renderOptionLabelCB,
    focusedOptionIndex: focusedOption.index
  })), isAriaLiveEnabled && React.createElement(AriaLiveRegion, {
    menuOpen: menuOpen,
    isFocused: isFocused,
    ariaLabel: ariaLabel,
    inputValue: inputValue,
    isSearchable: isSearchable,
    focusedOption: focusedOption,
    selectedOption: selectedOption,
    optionCount: menuOptions.length
  })));
});
Select.displayName = 'Select';

export { FilterMatchEnum, Select };
